<?php

namespace Teknisa\TCO\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DTO\Response\Message;
use Teknisa\TCO\Services\Tco00400TipoContratoService;

class Tco00400TipoContratoController extends \Teknisa\GER\Controllers\TeknisaCrud {
    
    protected $crudService;
    protected $environment;
    protected $entityManager;

    public function __construct($crudService, $parameterBag, $environment, $entityManager, Tco00400TipoContratoService $service) {
        $this->crudService = $crudService;
        $this->environment = $environment;
        $this->entityManager = $entityManager;
        $this->service = $service;
    }

    public function salvaNovoTipoContrato(DTO\Request $request, DTO\Response $response) {
        $tipoContrato = $request->getRow();
        
        if($this->isValidTipoContrato($tipoContrato, $response)){
            $tipoContrato["CDTIPOCONTRATO"] = str_pad($tipoContrato["CDTIPOCONTRATO"], 5, "0", STR_PAD_LEFT);
            $this->salvaTipoContrato($request);
            
            $response->addDataSet(new DataSet("tipocontrato", array()));
            $response->addDataSet(new DataSet("columnsToReturn", $this->crudService->getColumnsToReturn()));
        }
    }
    
    private function isValidTipoContrato($tipoContrato, $response) {
        $arrayErrors = array();
        
        if(empty($tipoContrato["CDTIPOCONTRATO"])){
            $response->addMessage(new Message('EMPTY_CODIGO'));
            return false;
        }
        if(strlen($tipoContrato["CDTIPOCONTRATO"]) > 5){
            $response->addMessage(new Message('TAMANHO_CODIGO'));
            return false;
        }
        if(preg_match('/^([0-9]*)$/', $tipoContrato["CDTIPOCONTRATO"]) < 1){
            $response->addMessage(new Message('CODIGO_INVALIDO'));
            return false;
        }
        if(strlen($tipoContrato["NMTIPOCONTRATO"]) > 50){
            $response->addMessage(new Message('TAMANHO_NOME'));
            return false;
        }

        return true;
    }

	private function salvaTipoContrato($request) {
        $tipoContrato = $this->crudService->getRowsToSave($request, "tipocontrato");
        $this->crudService->save("tipocontrato", $tipoContrato);
        $tipoContrato[0]["__is_new"] = false;
        $this->crudService->addColumnToReturn($tipoContrato[0]);
    }
    
    public function checkRemoveTipoContrato(DTO\Request $request, DTO\Response $response) {
        $tipoContrato = $request->getDataSet()->getRows()[0];
        $tipoContratoVinculados = $this->verificaRamosAtividadeUtilizados($tipoContrato); 
        
        if(empty($tipoContratoVinculados)){
            $this->removeTipoContrato($request, $response);
            $response->addDataSet(new DataSet("tipoContratoVinculados", array()));
        } else {
            $response->addDataSet(new DataSet("tipoContratoVinculados", $tipoContratoVinculados));
        }
    }
    
    private function verificaRamosAtividadeUtilizados($tipoContrato) {
        
        $params =  array(
            'NRTIPOCONTRATO'  => $tipoContrato->get('NRTIPOCONTRATO')
        );
        
        $retorno = $this->service->retornaTipoContratoVinculados($params);
        
        return $retorno;
    }
    
    private function removeTipoContrato($request, $response) {
		$rows = $this->crudService->getRowsToDelete($request, 'tipocontrato');
        $dataSet = new DataSet("tipocontrato", $rows);
		$this->crudService->delete($response, $dataSet);
	}
}
