<?php

namespace Teknisa\TCO\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DataSource;

class Tco00900ParametrosComerciais extends \Teknisa\GER\Controllers\TeknisaCrud {
    
    protected $crudService;
    protected $environment;
    protected $entityManager;
    protected $dataSourceManager;
    protected $Tco00900ParametrosComerciaisService;
    protected $newCode;
    protected $dataSourceToSave = "tco00900_paramtco";
    protected $dataSourceToSaveQtdRef = "tco00900_qtdPorRefeicao";
    protected $dataSourceToSaveLimitador = "tco00900_findLimitador";

    public function __construct($crudService, $parameterBag, $environment, $entityManager, $Tco00900ParametrosComerciaisService, $dataSourceManager, $newCode) {
        $this->crudService                          = $crudService;
        $this->environment                          = $environment;
        $this->entityManager                        = $entityManager;
        $this->Tco00900ParametrosComerciaisService  = $Tco00900ParametrosComerciaisService;
        $this->dataSourceManager                    = $dataSourceManager;
        $this->newCode                              = $newCode;
    }

    public function saveGeral(DTO\Request $request, DTO\Response $response) {
        try {
            $this->entityManager->beginTransaction();
            $dataSet = $request->getDataSet();
            if($this->validateEmail($dataSet)) throw new \Exception("Email inválido.");
            $rows = $dataSet->getRows();
            $rows[0]["__is_new"] = sizeof($this->Tco00900ParametrosComerciaisService->retornParamtco($rows[0])) == 0;
            $rows[0]['QTDIASVIGENPROPCOM'] = isset($rows[0]['QTDIASVIGENPROPCOM']) ? $rows[0]['QTDIASVIGENPROPCOM']: null;
            if($rows[0]["__is_new"]){
                $rows[0]['NRPARAMTCO'] = 1;
            }
            
            $newDataSet = new DataSet("paramtco", $rows);
            $newRequest = new DTO\Request\DataSet($dataSet, "", "", "");
            
            $rows = $this->crudService->getRowsToSave($request, "paramtco");

            $this->crudService->save("paramtco", $rows);
            
            $this->entityManager->commit();
            
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
        
        
        // try {
        //     $dataSet = $request->getDataSet();
        //     if($this->validateEmail($dataSet)) throw new \Exception("Email inválido.");
        //     $dataSet = $this->Tco00900ParametrosComerciaisService->verifyExistParamTco($dataSet);
        //     $this->verifyFilialHasChanged($dataSet->getRows());
        //     $dataSet = $this->defineDefaultValues($dataSet);
        //     $dataSet->setDataSourceName($this->dataSourceToSave);
        //     $persistedRows = $this->dataSourceManager->persist($dataSet);
        //     $response->addDataSet(new DataSet($this->dataSourceToSave, $persistedRows));
        // } catch (\Exception $e) {
        //     $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        // }
    }
    
    public function validateEmail($dataSet){
        $email = empty($dataSet->getRows()[0]["DSEMAILNOTIFICACAO"]) ? "" : $dataSet->getRows()[0]["DSEMAILNOTIFICACAO"]; 
        if(!empty($email)){
            if(filter_var($email, FILTER_VALIDATE_EMAIL)){
                return false;
            }else{
                return true;
            }
        }else{
            return false;
        }
    }
    
    public function saveLimitadorCustos(DTO\Request $request, DTO\Response $response) {
        try {
            $dataSet = $request->getDataSet();
            $dataSet = $this->defineDefaultValues($dataSet);
            $dataSet->getRows()[0]["VRLIMITECUSTO"] = $dataSet->getRows()[0]["VRLIMITECUSTO"];
            $dataSet = $this->Tco00900ParametrosComerciaisService->verifyExistParamTco($dataSet);
            $dataSet->setDataSourceName($this->dataSourceToSaveLimitador);
            $persistedRows = $this->dataSourceManager->persist($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceToSaveLimitador, $persistedRows));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function verifyFilialHasChanged($dataSet){
        $cdfilial = empty($dataSet[0]['CDFILIAL']) ? "" : $dataSet[0]['CDFILIAL'];
        if($this->Tco00900ParametrosComerciaisService->verifyFilialHasChanged($cdfilial))
            $this->Tco00900ParametrosComerciaisService->updateServicoProposta();
    }
    
     public function defineDefaultValues($dataSet){
        $data = $dataSet->getRows()[0];
        $data["__is_new"] = false;
        $data["NRORG"] = $this->environment->getNrOrg();
        $data["NRPARAMTCO"] = 1;
        if(empty($data["NRPERFILACESSO"])) $data["NRPERFILACESSO"] = "";
        if(empty($data["CDFILIAL"])) $data["CDFILIAL"] = "";
        if(empty($data["CDTIPOCUSTO"])) $data["CDTIPOCUSTO"] = "";
        if(empty($data["DSSERVIDORSMTP"])) $data["DSSERVIDORSMTP"] = "";
        if(empty($data["DSUSUARIOSMTP"])) $data["DSUSUARIOSMTP"] = "";
        if(empty($data["DSSENHASMTP"])) $data["DSSENHASMTP"] = "";
        if(empty($data["DSEMAILNOTIFICACAO"])) $data["DSEMAILNOTIFICACAO"] = "";
        if(empty($data["CDTIPOPRATLIMPEZA"])) $data["CDTIPOPRATLIMPEZA"] = "";
        if(empty($data["CDGRUPPROD"])) $data["CDGRUPPROD"] = "";
        return $dataSet;
    }
    
    public function saveQtdPorRefeicao(DTO\Request $request, DTO\Response $response) {
        try {
            $dataSet = $request->getDataSet();
            if($this->verifyIsNumber($dataSet->getRows()[0])) throw new \Exception("Valor numérico inválido.");
            if($dataSet->getRows()[0]["QTINICIAL"] < 0) throw new \Exception("Quantidade Inicial não pode ser negativa.");
            if($dataSet->getRows()[0]["QTFINAL"] < 0) throw new \Exception("Quantidade Final não pode ser negativa.");
            if($dataSet->getRows()[0]["QTSERVICO"] < 0) throw new \Exception("N° Serviços não pode ser negativo.");
            if($dataSet->getRows()[0]["QTINICIAL"] > $dataSet->getRows()[0]["QTFINAL"]) 
                throw new \Exception("Quantidade Inicial não pode ser maior que a Quantidade Final.");
            $result = $this->Tco00900ParametrosComerciaisService->checkIntervalIsValid($dataSet->getRows()[0]);
            if(!empty($result))
                throw new \Exception("Intervalo informado sobre outro intervalo já cadastrado.");
            $dataSet = $this->Tco00900ParametrosComerciaisService->verifyExistParamTco($dataSet);
            $dataSet->getRows()[0]["NRORG"] = $this->environment->getNrOrg();
            $dataSet->getRows()[0]["NRPARAMTCO"] = 1;
            if(empty($dataSet->getRows()[0]["NRSERCOMPREFEICAO"]))
                $dataSet->getRows()[0]["NRSERCOMPREFEICAO"] = $this->newCode->getCodeByTableName('NRSERCOMPREFEICAO', $dataSet->getRows()[0]["NRORG"]);
            $dataSet->setDataSourceName($this->dataSourceToSaveQtdRef);
            $persistedRows = $this->dataSourceManager->persist($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceToSaveQtdRef, $persistedRows));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function verifyIsNumber($row){
        if(ctype_digit((string)$row["QTINICIAL"]) && ctype_digit((string)$row["QTINICIAL"]) && ctype_digit((string)$row["QTINICIAL"]))
            return false;
        else 
            return true;
    }
    
    public function removeQtdPorRefeicao(DTO\Request $request, DTO\Response $response){
        try {
            $rows = $this->crudService->getRowsToDelete($request, 'tco00900_qtdPorRefeicao');
            $dataSet = new DataSet("tco00900_qtdPorRefeicao", $rows);
    		$this->crudService->delete($response, $dataSet);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
}